package com.ruoyi.ss.bonus.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.bo.RechargeBO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author wjh
 * 2024/8/20
 */
@Service
public class BonusConverterImpl implements BonusConverter {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private TransactionBillService transactionBillService;

    /**
     * 订单转为分成明细
     *
     * @param bo
     */
    @Override
    public List<Bonus> toPo(RechargeBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        TransactionBill bill = bo.getOrder();
        if (bill == null) {
            return null;
        }

        List<Bonus> result = bo.getBonusList();
        for (Bonus bonus : result) {
            bonus.setBillId(bill.getBillId());
            bonus.setBillNo(bill.getBillNo());
        }

        return result;
    }

    @Override
    public List<Bonus> genBonusList(SmUserVo mch, SmUserVo agent, SysDept platform, DeviceVO device) {
        List<Bonus> result = new ArrayList<>();

        BigDecimal point = BigDecimal.valueOf(100);

        // TODO 平台收取服务费

        // TODO 代理商收取服务费

        // TODO 剩余的给商户

//
//        // 根据设备的投资人、经营场所获取各自的分成比例
//        result.add(this.toPo(store, storeInvestor, mch));
//        result.add(this.toPo(mch, mch.getPoint()));
//        BigDecimal lastPoint = mch.getPoint().add(storeInvestor.getPoint());   // 上一次的分成比例
//
//        // 投资人上级
//        if (StringUtils.hasText(mch.getAncestors()) && CollectionUtils.isNotEmpty(agent)) {
//            List<Long> collect = Arrays.stream(mch.getAncestors().split(",")).map(Long::valueOf).collect(Collectors.toList());
//            for (int i = collect.size() - 1; i >= 0; i --) {
//                Long id = collect.get(i);
//                SmUserVo investorAncestor = agent.stream().filter(item -> item.getUserId().equals(id)).findFirst().orElse(null);
//                if (investorAncestor != null) {
//                    result.add(this.toPo(investorAncestor, investorAncestor.getPoint().subtract(lastPoint)));
//                    lastPoint = investorAncestor.getPoint();
//                }
//            }
//        }
//
//        // 直属部门
//        result.add(this.toPo(platform, platform.getPoint().subtract(lastPoint)));
//        lastPoint = platform.getPoint();
//
//        // 直属部门的祖级列表
//        if (StringUtils.hasText(platform.getAncestors()) && CollectionUtils.isNotEmpty(investorDeptList)) {
//            List<Long> collect = Arrays.stream(platform.getAncestors().split(",")).map(Long::valueOf).collect(Collectors.toList());
//            for (int i = collect.size() - 1; i >= 0; i --) {
//                Long id = collect.get(i);
//                SysDept d = investorDeptList.stream().filter(item -> item.getDeptId().equals(id)).findFirst().orElse(null);
//                if (d != null) {
//                    result.add(this.toPo(d, d.getPoint().subtract(lastPoint)));
//                    lastPoint = d.getPoint();
//                }
//            }
//        }

        // 校验分成比例
//        ServiceUtil.assertion(lastPoint.compareTo(BigDecimal.valueOf(100)) != 0, "分成计算失败：最终分成比例不等于100");
//        BigDecimal total = BigDecimal.ZERO;
//        BigDecimal decimal100 = BigDecimal.valueOf(100);
//        for (Bonus bonus : result) {
//            ServiceUtil.assertion(bonus.getPoint().compareTo(BigDecimal.ZERO) < 0, "分成计算失败：分成比例小于0");
//            ServiceUtil.assertion(bonus.getPoint().compareTo(decimal100) > 0, "分成计算失败：分成比例大于100");
//            total = total.add(bonus.getPoint());
//        }
//        ServiceUtil.assertion(total.compareTo(decimal100) > 0, "分成计算失败：分成比例总和大于100");

        return result;
    }

    private Bonus toPo(SysDept dept, BigDecimal point) {
        if (dept == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(dept.getDeptId());
        po.setArrivalName(dept.getDeptName());
        BonusArrivalType arrivalType = BonusArrivalType.parseByDept(dept);
        if (arrivalType != null) {
            po.setArrivalType(arrivalType.getType());
        }
        po.setPoint(point);
        po.setAncestors(dept.getAncestors());
        po.setDeptId(dept.getDeptId());

        return po;
    }

    private Bonus toPo(SmUserVo user, BigDecimal point, DeviceVO device) {
        if (user == null || point == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(user.getUserId());
        po.setArrivalName(user.getRealOrUserName());
        BonusArrivalType arrivalType = BonusArrivalType.parseByUserType(user.getType());
        if (arrivalType != null) {
            po.setArrivalType(arrivalType.getType());
        }
        po.setPoint(point);
//        po.setAncestors(Arrays.asList(device.getAgentId(), device.getUserId()));
        po.setDeptId(Constants.ROOT_DEPT);

        return po;
    }
}
