package com.ruoyi.ss.bonus.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.domain.vo.CommonCountVO;
import com.ruoyi.common.domain.vo.CommonSumVO;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.bonus.domain.BonusProvideQuery;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.domain.vo.BonusDailyAmountVO;
import com.ruoyi.ss.bonus.domain.vo.BonusMonthAmountVO;
import com.ruoyi.ss.bonus.domain.vo.ProvideBonusVO;
import com.ruoyi.ss.recordBalance.domain.enums.RecordBalanceBstType;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.ss.bonus.mapper.BonusMapper;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.BonusVO;
import com.ruoyi.ss.bonus.domain.BonusQuery;
import com.ruoyi.ss.bonus.service.BonusService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 分成明细Service业务层处理
 *
 * @author ruoyi
 * @date 2024-08-20
 */
@Service
public class BonusServiceImpl implements BonusService
{
    @Autowired
    private BonusMapper bonusMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private StoreService storeService;

    /**
     * 查询分成明细
     *
     * @param id 分成明细主键
     * @return 分成明细
     */
    @Override
    public BonusVO selectBonusById(Long id)
    {
        return bonusMapper.selectBonusById(id);
    }

    /**
     * 查询分成明细列表
     *
     * @param bonus 分成明细
     * @return 分成明细
     */
    @Override
    public List<BonusVO> selectBonusList(BonusQuery bonus)
    {
        return bonusMapper.selectBonusList(bonus);
    }

    /**
     * 新增分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int insertBonus(Bonus bonus)
    {
        this.buildBeforeCreate(bonus);
        return bonusMapper.insertBonus(bonus);
    }

    // 补全新增前的数据
    private void buildBeforeCreate(Bonus bonus) {
        bonus.setCreateTime(DateUtils.getNowDate());
    }

    /**
     * 修改分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int updateBonus(Bonus bonus)
    {
        return bonusMapper.updateBonus(bonus);
    }

    /**
     * 批量删除分成明细
     *
     * @param ids 需要删除的分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusByIds(Long[] ids)
    {
        return bonusMapper.deleteBonusByIds(ids);
    }

    /**
     * 删除分成明细信息
     *
     * @param id 分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusById(Long id)
    {
        return bonusMapper.deleteBonusById(id);
    }

    @Override
    public int batchInsert(List<Bonus> bonusList) {
        if (CollectionUtils.isEmptyElement(bonusList)) {
            return 0;
        }
        for (Bonus bonus : bonusList) {
            this.buildBeforeCreate(bonus);
        }
        return bonusMapper.batchInsert(bonusList);
    }

    @Override
    public int payBonus(List<BonusVO> bonusList, BigDecimal money) {
        if (CollectionUtils.isEmptyElement(bonusList)) {
            return 0;
        }

        BigDecimal decimal100 = new BigDecimal(100);
        BigDecimal dividedAmount = BigDecimal.ZERO; // 已分配金额
        LocalDateTime now = LocalDateTime.now();

        // 循环遍历，构造分成金额
        for (BonusVO bonus : bonusList) {
            BigDecimal amount = money.multiply(bonus.getPoint()).divide(decimal100, 2, RoundingMode.HALF_UP);
            bonus.setAmount(amount);
            bonus.setStatus(BonusStatus.DIVIDEND.getStatus());
            bonus.setPayTime(now);
            dividedAmount = dividedAmount.add(amount);
        }
        // 平台吃掉误差
        BonusVO platform = bonusList.stream().filter(bonus -> bonus.getArrivalType().equals(BonusArrivalType.PLATFORM.getType())).findFirst().orElse(null);
        ServiceUtil.assertion(platform == null, "平台不存在");
        if (dividedAmount.compareTo(money) != 0) {
            BigDecimal subtract = money.subtract(dividedAmount);    // 误差值
            platform.setAmount(platform.getAmount().add(subtract));
        }

        Integer result = transactionTemplate.execute(status -> {

            // 循环遍历，添加金额到账户上
            for (BonusVO bonus : bonusList) {
                int add = 0;
                // 根据类型，添加金额
                if (BonusArrivalType.userList().contains(bonus.getArrivalType())) {
                    add = userService.addBalance(bonus.getArrivalId(), bonus.getAmount(), String.format("订单分成:%s", bonus.getBillNo()), RecordBalanceBstType.RECHARGE, bonus.getBillId());
                } else if (BonusArrivalType.deptList().contains(bonus.getArrivalType())) {
//                    add = deptService.addBalance(bonus.getArrivalId(), bonus.getAmount(), String.format("订单分成:%s", bonus.getBillNo()), RecordBalanceBstType.RECHARGE, bonus.getBillId());
                }
                ServiceUtil.assertion(add != 1, "增加账户金额失败");
            }

            // 批量更新分成金额
            int updateAmount = this.batchUpdateAmount(bonusList);
            ServiceUtil.assertion(updateAmount != bonusList.size(), "更新分成金额失败");

            return updateAmount;
        });

        return result == null ? 0 : result;
    }

    @Override
    public List<CommonCountVO<Long>> selectCountByArrival(BonusQuery query) {
        return bonusMapper.selectCountByArrival(query);
    }

    @Override
    public List<CommonSumVO<Long>> selectBillAmountByArrival(BonusQuery query) {
        return bonusMapper.selectBillAmountByArrival(query);
    }

    @Override
    public List<ProvideBonusVO> selectProvideBonus(BonusProvideQuery query) {
        return bonusMapper.selectProvideBonus(query);
    }

    @Override
    public int addRefundAmount(Long id, BigDecimal amount) {
        if (id == null) {
            return 0;
        }
        ServiceUtil.assertion(amount.compareTo(BigDecimal.ZERO) < 0, "退款金额必须大于0");
        return bonusMapper.addRefundAmount(id, amount);
    }

    @Override
    public <T extends Bonus> List<T> filterBonusScope(List<T> bonusList) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        Long userId = loginUser.getUserId();
        Long deptId = loginUser.getDeptId();
        return bonusList.stream().filter(bonus -> {
            // 前台用户过滤
            if (LoginType.FRONT.equals(loginUser.getLoginType())) {
                return BonusArrivalType.userList().contains(bonus.getArrivalType()) && (
                        Arrays.asList(bonus.getAncestors().split(",")).contains(userId.toString())
                        || bonus.getArrivalId().equals(userId)
                );
            }
            // 后台用户过滤
            else if (LoginType.ADMIN.equals(loginUser.getLoginType())) {
                if (BonusArrivalType.userList().contains(bonus.getArrivalType())) {
                    return true;
                }
                if (BonusArrivalType.deptList().contains(bonus.getArrivalType())) {
                    return Arrays.asList(bonus.getAncestors().split(",")).contains(deptId.toString())
                            || bonus.getArrivalId().equals(deptId);
                }
            }
            return false;
        }).collect(Collectors.toList());
    }

    @Override
    public List<BonusMonthAmountVO> selectMonthAmount(BonusQuery query) {
        return bonusMapper.selectMonthAmount(query);
    }

    @Override
    public List<BonusDailyAmountVO> selectDailyAmount(BonusQuery query) {
        return bonusMapper.selectDailyAmount(query);
    }

    private int batchUpdateAmount(List<BonusVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        return bonusMapper.batchUpdateAmount(list);
    }
}
