package com.ruoyi.ss.deviceBindRecord.service;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.deviceBindRecord.domain.SmDeviceBindRecord;
import com.ruoyi.ss.deviceBindRecord.mapper.SmDeviceBindRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 设备绑定记录Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-17
 */
@Service
public class SmDeviceBindRecordServiceImpl implements ISmDeviceBindRecordService
{
    @Autowired
    private SmDeviceBindRecordMapper smDeviceBindRecordMapper;

    /**
     * 查询设备绑定记录
     *
     * @param recordId 设备绑定记录主键
     * @return 设备绑定记录
     */
    @Override
    public SmDeviceBindRecord selectSmDeviceBindRecordByRecordId(Long recordId)
    {
        return smDeviceBindRecordMapper.selectSmDeviceBindRecordByRecordId(recordId);
    }

    /**
     * 查询设备绑定记录列表
     *
     * @param smDeviceBindRecord 设备绑定记录
     * @return 设备绑定记录
     */
    @Override
    public List<SmDeviceBindRecord> selectSmDeviceBindRecordList(SmDeviceBindRecord smDeviceBindRecord)
    {
        return smDeviceBindRecordMapper.selectSmDeviceBindRecordList(smDeviceBindRecord);
    }

    /**
     * 新增设备绑定记录
     *
     * @param smDeviceBindRecord 设备绑定记录
     * @return 结果
     */
    @Override
    public int insertSmDeviceBindRecord(SmDeviceBindRecord smDeviceBindRecord)
    {
        smDeviceBindRecord.setCreateTime(DateUtils.getNowDate());
        return smDeviceBindRecordMapper.insertSmDeviceBindRecord(smDeviceBindRecord);
    }

    /**
     * 修改设备绑定记录
     *
     * @param smDeviceBindRecord 设备绑定记录
     * @return 结果
     */
    @Override
    public int updateSmDeviceBindRecord(SmDeviceBindRecord smDeviceBindRecord)
    {
        return smDeviceBindRecordMapper.updateSmDeviceBindRecord(smDeviceBindRecord);
    }

    /**
     * 批量删除设备绑定记录
     *
     * @param recordIds 需要删除的设备绑定记录主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceBindRecordByRecordIds(Long[] recordIds)
    {
        return smDeviceBindRecordMapper.deleteSmDeviceBindRecordByRecordIds(recordIds);
    }

    /**
     * 删除设备绑定记录信息
     *
     * @param recordId 设备绑定记录主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceBindRecordByRecordId(Long recordId)
    {
        return smDeviceBindRecordMapper.deleteSmDeviceBindRecordByRecordId(recordId);
    }

    @Override
    public int record(Long userId, Long deviceId) {
        SmDeviceBindRecord record = new SmDeviceBindRecord();
        record.setUserId(userId);
        record.setDeviceId(deviceId);
        return this.insertSmDeviceBindRecord(record);
    }
}
