package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.bo.RefundSuccessBO;
import com.ruoyi.ss.payBill.domain.dto.PayBillRefundDTO;
import com.ruoyi.ss.payBill.interfaces.AfterPay;
import com.ruoyi.ss.payBill.interfaces.AfterRefund;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.recordBalance.domain.enums.RecordBalanceBstType;
import com.ruoyi.ss.suit.domain.enums.SuitFeeType;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.bo.PaySuccessBO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;

/**
 * 充值成功处理类
 * @author wjh
 * 2024/8/14
 */
@Service
public class RechargePayHandler implements AfterPay, AfterRefund {


    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private PayBillService payBillService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ISmUserService userService;

    @Override
    public int onPaySuccess(PayBillVO payBill) {
        TransactionBillVO bill = transactionBillService.selectSmTransactionBillByBillId(payBill.getBstId());
        if (bill == null) {
            return 0;
        }

        PaySuccessBO bo = new PaySuccessBO();
        bo.setOrder(bill);
        bo.setPayBill(payBill);

        if (SuitFeeType.singleList().contains(bill.getSuitFeeType())) {
            return transactionBillService.rechargeSuccess(bo);
        } else {
            return this.handleTimingPaySuccess(bo);
        }
    }

    /**
     * 处理分时段订单支付成功
     */
    private int handleTimingPaySuccess(PaySuccessBO bo) {
        if (bo == null) {
            return 0;
        }

        TransactionBillVO order = bo.getOrder();
        PayBillVO pay = bo.getPayBill();

        ServiceUtil.assertion(order == null, "充值订单不存在");
        ServiceUtil.assertion(!TransactionBillStatus.UNPAID.getStatus().equals(order.getStatus()), "当前订单状态非待支付状态");

        Integer result = transactionTemplate.execute(status -> {
            // 修改订单状态
            TransactionBill data = new TransactionBill();
            data.setPayTime(DateUtils.toDate(pay.getPayTime()));
            data.setPayId(pay.getPayId());
            data.setChannelCost(pay.getChannelCost());
            data.setChannelId(pay.getChannelId());
            data.setStatus(TransactionBillStatus.SUCCESS.getStatus());
            TransactionBillQuery query = new TransactionBillQuery();
            query.setBillId(order.getBillId());
            query.setStatus(TransactionBillStatus.UNPAID.getStatus());
            int update = transactionBillService.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "修改订单状态失败");

            // 商户余额增加
            userService.addBalance(order.getMchId(), order.getArrivalAmount(), StringUtils.format("分时段订单%s", order.getBillNo()), RecordBalanceBstType.RECHARGE, order.getBillId());

            // 如果有押金，则申请押金退款
            if (order.getSuitPrice() != null && order.getSuitPrice().compareTo(BigDecimal.ZERO) > 0 && order.getDepositPayId() != null) {
                // 押金退款
                PayBillRefundDTO refundDto = new PayBillRefundDTO();
                refundDto.setPayId(order.getDepositPayId());
                refundDto.setRefundAmount(order.getSuitPrice());
                refundDto.setRefundReason(String.format("充值订单%s押金退款", order.getBillNo()));
                int refund = payBillService.refund(refundDto);
                ServiceUtil.assertion(refund != 1, "申请退款失败");
            }

            return update;
        });

        return result == null ? 0 : result;

    }

    @Override
    public int onRefundSuccess(RefundSuccessBO bo) {
        PayBillVO pay = bo.getPay();

        Integer result = transactionTemplate.execute(status -> {
            TransactionBill billData = new TransactionBill();
            billData.setStatus(TransactionBillStatus.REFUNDED.getStatus());
            TransactionBillQuery billQuery = new TransactionBillQuery();
            billQuery.setBillId(pay.getBstId());
            billQuery.setStatus(TransactionBillStatus.REFUNDING.getStatus());
            int updateBill = transactionBillService.updateByQuery(billData, billQuery);
            ServiceUtil.assertion(updateBill != 1, "修改原订单状态失败");

            return updateBill;
        });

        return result == null ? 0 : result;
    }
}
