package com.ruoyi.ss.user.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.ss.user.domain.SmUserQuery;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.user.service.UserValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashSet;
import java.util.List;

/**
 * @author wjh
 * 2024/4/29
 */
@Service
public class UserValidatorImpl extends BaseValidator implements UserValidator {

    @Autowired
    private ISmUserService userService;

    /**
     * 判断用户是否存在
     *
     * @param userIds 用户id列表
     */
    @Override
    public boolean isExist(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return true;
        }
        SmUserQuery query = new SmUserQuery();
        query.setUserIds(userIds);
        Integer userCount = userService.selectCount(query);
        return userCount == new HashSet<Long>(userIds).size();
    }

    /**
     * 判断用户是否是商家
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isMch(Long userId) {
        if (userId == null) {
            return false;
        }
        SmUserQuery query = new SmUserQuery();
        query.setUserId(userId);
        query.setIsMch(true);
        return userService.selectCount(query) > 0;
    }

    /**
     * 判断用户是否可用
     *
     * @param userId
     */
    @Override
    public boolean isUsage(Long userId) {
        if (userId == null) {
            return false;
        }
        SmUserVo user = userService.selectSmUserByUserId(userId);
        if (user == null || UserStatus.DELETED.getCode().equals(user.getDelFlag()) || UserStatus.DISABLE.getCode().equals(user.getStatus())) {
            return false;
        }
        return true;
    }

    /**
     * 逻辑删除前校验
     */
    @Override
    public ValidateResult preLogicDel(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return error("用户id不能为空");
        }
        if (!isExist(userIds)) {
            return error("用户不存在");
        }
        return success();
    }
}
